@@ -159,7 +159,7 @@ static struct mpi_p2p_msg_t* __mpi_send_generic(const char* thread_id, int src,
   struct mpi_p2p_msg_t* msg = __start_send_message(CUR_TIME(CUR_INDEX), src, actual_dest, len, tag, thread_id, mpi_req);
   assert(msg);
 
-  if(! msg->times[stop_recv]) {
+  if(! IS_TIME_SET(msg->times[stop_recv])) {
 #ifndef GTG_OUT_OF_ORDER
     MPI_CHANGE() startLink(CURRENT, "L_MPI_P2P", "C_Prog",
 	msg->sender_thread_id, msg->recver_thread_id,
@@ -233,7 +233,7 @@ __mpi_stop_recv_generic (uint64_t *timestamp,
     return 1;
   }
 
-  if(msg->times[start_isend]) {
+  if(IS_TIME_SET(msg->times[start_isend])) {
     if(msg->times[start_isend] > msg->times[stop_recv]) {
       msg->times[stop_recv] += add_delay_to_trace(rank_to_trace_id[dest],
 	  msg->times[stop_recv],
@@ -260,6 +260,7 @@ __mpi_stop_recv_generic (uint64_t *timestamp,
 	msg->sender_thread_id, thread_id,
 	msg->link_value, msg->id);
 
+#if 0
     char* message_str = NULL;
     float throughput = -1;
     /* duration in us */
@@ -281,6 +282,7 @@ __mpi_stop_recv_generic (uint64_t *timestamp,
     }
 
     MPI_CHANGE() addEvent((NS_TO_MS(*timestamp)), "E_MPI_EndComm", thread_id, message_str);
+#endif
 
 #else  /* GTG_OUT_OF_ORDER */
     MPI_CHANGE() endLink((NS_TO_MS(*timestamp)), "L_MPI_P2P", "C_Prog",
@@ -312,7 +314,7 @@ static void __mpi_barrier_start_generic(struct mpi_coll_msg_t* msg, int my_rank)
 
 #ifdef GTG_OUT_OF_ORDER
 	assert(msg->thread_ids[my_rank]);
-	if(msg->times[i][stop_coll]) {
+	if(IS_TIME_SET(msg->times[i][stop_coll])) {
 	  /* the remote process stop_coll already happened.
 	   * This means that there's a synchronisation problem between the traces
 	   */
@@ -343,14 +345,14 @@ static int __mpi_barrier_stop_generic(struct mpi_coll_msg_t* msg, int my_rank)
 
   for(i=0; i<msg->comm[my_rank]->comm_size; i++) {
     if(i != my_rank) {
-      if(! msg->times[i][start_coll]) {
+      if(!IS_TIME_SET(msg->times[i][start_coll])) {
 	/* ith process has not yet reached the MPI_Barrier.
 	 * This means there's a synchronisation problem between the traces.
 	 * Let's wait until the remote process starts MPI_Barrier
 	 */
 	SKIP = 1;
 	CUR_TRACE->skip = 1;
-	msg->times[my_rank][stop_coll] = 0;
+	msg->times[my_rank][stop_coll] = TIME_INIT;
 	return 1;
       }
     }
@@ -384,8 +386,14 @@ handle_mpi_init ()
   GET_PARAM_PACKED_6(CUR_EV, rank, world_size, mpi_any_source, mpi_any_tag, mpi_request_null, mpi_proc_null);
   EZT_CONVERT_PRINTF(1, "MPI_Init(rank=%d, world_size=%d, mpi_any_source=%x, mpi_any_tag=%x, mpi_request_null=%x, mpi_proc_null=%d)\n", rank, world_size, mpi_any_source, mpi_any_tag, mpi_request_null, mpi_proc_null);
 
+  int *nb_traces = get_nb_traces();
+  if(*nb_traces < world_size) {
+    fprintf(stderr, "Error: the size of MPI_COMM_WORLD is %d, but only %d trace files were provided. Please give me all the trace files !\n", world_size, *nb_traces);
+    abort();
+  }
+
   if(!rank_to_trace_id) {
-    int *nb = get_nb_traces();
+    int *nb = nb_traces;
     rank_to_trace_id = calloc(*nb, sizeof(int));
   }
 
@@ -512,7 +520,7 @@ void __ezt_mpi_enter_function(MPI_id_t id) {
   DECLARE_CUR_THREAD(p_thread);
   INIT_MPI_THREAD_INFO(p_thread, thread_info);
 
-  assert(thread_info->enter_date_mpi_calls[id] == 0);
+  assert(thread_info->enter_date_mpi_calls[id] < 0);
 
   thread_info->nb_mpi_calls[id]++;
   thread_info->enter_date_mpi_calls[id] = CURRENT;
@@ -522,14 +530,14 @@ void __ezt_mpi_leave_function(MPI_id_t id) {
   DECLARE_CUR_THREAD(p_thread);
   INIT_MPI_THREAD_INFO(p_thread, thread_info);
 
-  assert(thread_info->enter_date_mpi_calls[id] != 0);
-  assert(thread_info->leave_date_mpi_calls[id] == 0);
+  assert(thread_info->enter_date_mpi_calls[id] >= 0);
+  assert(thread_info->leave_date_mpi_calls[id] < 0);
 
   thread_info->leave_date_mpi_calls[id] = CURRENT;
   thread_info->total_time_mpi_calls[id] += thread_info->leave_date_mpi_calls[id]-thread_info->enter_date_mpi_calls[id];
 
-  thread_info->enter_date_mpi_calls[id] = 0;
-  thread_info->leave_date_mpi_calls[id] = 0;
+  thread_info->enter_date_mpi_calls[id] = -1;
+  thread_info->leave_date_mpi_calls[id] = -1;
 }
 
 void
@@ -548,10 +556,10 @@ handle_mpi_start_send ()
 
   struct mpi_p2p_msg_t* msg = __mpi_send_generic( thread_id, CUR_RANK, dest, len, tag, NULL, comm);
 
-  if(!msg->times[stop_isend])
+  if(!IS_TIME_SET(msg->times[stop_isend]))
     msg->times[stop_isend] = CUR_TIME(CUR_INDEX);
 
-  if(!msg->times[start_swait])
+  if(!IS_TIME_SET(msg->times[start_swait]))
     msg->times[start_swait] = CUR_TIME(CUR_INDEX);
 
   MPI_CHANGE() pushState(CURRENT, "ST_Thread", thread_id, "STV_MPI_Send");
@@ -604,12 +612,15 @@ handle_mpi_start_recv ()
   struct mpi_p2p_msg_t* msg = __start_recv_message(CUR_TIME(CUR_INDEX),
       actual_src, dest, len, tag, thread_id, NULL);
 
-  if(!msg->times[stop_irecv])
+  if(!IS_TIME_SET(msg->times[stop_irecv]))
     msg->times[stop_irecv] = CUR_TIME(CUR_INDEX);
 
-  if(!msg->times[start_rwait])
+  if(!IS_TIME_SET(msg->times[start_rwait]))
     msg->times[start_rwait] = CUR_TIME(CUR_INDEX);
 
+#if DEBUG_MSG_MATCHING
+  printf("Start recv(src=%x, dest=%x, tag=%x, len=%d, comm=%p\n", src, dest, tag, len, comm_id);
+#endif
   MPI_CHANGE() pushState(CURRENT, "ST_Thread", thread_id, "STV_MPI_Recv");
 }
 
@@ -622,10 +633,14 @@ handle_mpi_stop_recv ()
 
   int len;
   int src;
+  int dest = CUR_RANK;
   uint32_t tag;
   app_ptr comm;
   GET_PARAM_PACKED_4(CUR_EV, len, src, tag, comm);
   uint64_t timestamp = CUR_TIME(CUR_INDEX);
+#if DEBUG_MSG_MATCHING
+  printf("Stop recv(src=%x, dest=%x, tag=%x, len=%d, comm=%p\n", src, dest, tag, len, comm);
+#endif
   if(__mpi_stop_recv_generic (&timestamp, thread_id, src, CUR_RANK, len, tag, NULL, comm)) {
     /* waiting for a mpi_send that matches */
     return;
@@ -667,10 +682,10 @@ void handle_mpi_start_sendrecv ()
   /* process the send */
   if (ssrc != p_info->__MPI_PROC_NULL && sdest != p_info->__MPI_PROC_NULL) {
       msg = __mpi_send_generic( thread_id, ssrc, sdest, slen, stag, NULL, comm);
-      if(!msg->times[stop_isend]) {
+      if(!IS_TIME_SET(msg->times[stop_isend])) {
 	msg->times[stop_isend] = CUR_TIME(CUR_INDEX);
       }
-      if(!msg->times[start_swait]) {
+      if(!IS_TIME_SET(msg->times[start_swait])) {
 	msg->times[start_swait] = CUR_TIME(CUR_INDEX);
       }
   }
@@ -683,9 +698,9 @@ void handle_mpi_start_sendrecv ()
     }
     msg = __start_recv_message(CUR_TIME(CUR_INDEX),
 			       actual_src, rdest, rlen, rtag, thread_id, NULL);
-    if(!msg->times[stop_irecv])
+    if(!IS_TIME_SET(msg->times[stop_irecv]))
       msg->times[stop_irecv] = CUR_TIME(CUR_INDEX);
-    if(!msg->times[start_rwait])
+    if(!IS_TIME_SET(msg->times[start_rwait]))
       msg->times[start_rwait] = CUR_TIME(CUR_INDEX);
   }
   MPI_CHANGE() pushState(CURRENT, "ST_Thread", thread_id, "STV_MPI_Sendrecv");
@@ -848,7 +863,7 @@ void handle_mpi_stop_isend_generic() {
   struct mpi_p2p_msg_t* msg = __mpi_find_p2p_message_by_mpi_req(CUR_RANK,
 								mpi_req);
   assert(msg);
-  if(!msg->times[stop_isend])
+  if(!IS_TIME_SET(msg->times[stop_isend]))
       msg->times[stop_isend] = CUR_TIME(CUR_INDEX);
 
   /* todo: add a send event here */
@@ -915,7 +930,7 @@ void handle_mpi_stop_irecv()
   struct mpi_p2p_msg_t* msg = __mpi_find_p2p_message_by_mpi_req(CUR_RANK,
 								mpi_req);
   assert(msg);
-  if(!msg->times[stop_irecv])
+  if(!IS_TIME_SET(msg->times[stop_irecv]))
       msg->times[stop_irecv] = CUR_TIME(CUR_INDEX);
 
   /* todo: add a recv event here */
@@ -1294,7 +1309,7 @@ void handle_mpi_stop_waitany()
 
       if(req->req_type == mpi_req_recv) {
 	/*  this is a receive request */
-	msg->times[start_rwait] = 0;
+	msg->times[start_rwait] = TIME_INIT;
       } else {
 	/*  this is a send request */
 	msg->times[start_swait] = timestamp;
@@ -1398,11 +1413,13 @@ void handle_mpi_start_BCast()
   app_ptr comm_id;
   int my_rank;
   int comm_size;
-  int __attribute__((unused)) root;
-  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, root);
+  int data_size;
+  int root;
+  GET_PARAM_PACKED_5(CUR_EV, comm_id, comm_size, my_rank, root, data_size);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_bcast,
-      comm, my_rank, NULL, thread_id);
+					    comm, my_rank, data_size, NULL, thread_id);
+  msg->root_process = root;
   __mpi_barrier_start_generic(msg, my_rank);
 }
 
@@ -1416,14 +1433,16 @@ void handle_mpi_start_Gather()
   app_ptr comm_id;
   int my_rank;
   int comm_size;
-  GET_PARAM_PACKED_3(CUR_EV, comm_id, comm_size, my_rank);
+  int data_size;
+  int root;
+  GET_PARAM_PACKED_5(CUR_EV, comm_id, comm_size, my_rank, data_size, root);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
 
   assert(comm->my_rank == my_rank);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_gather,
-      comm, my_rank, NULL, thread_id);
-
+					    comm, my_rank, data_size, NULL, thread_id);
+  msg->root_process = root;
   __mpi_barrier_start_generic(msg, my_rank);
 
 }
@@ -1444,13 +1463,15 @@ void handle_mpi_start_Scatter()
   app_ptr comm_id;
   int my_rank;
   int comm_size;
-  GET_PARAM_PACKED_3(CUR_EV, comm_id, comm_size, my_rank);
+  int data_size;
+  int root;
+  GET_PARAM_PACKED_5(CUR_EV, comm_id, comm_size, my_rank, data_size, root);
 
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_scatter,
-					    comm, my_rank, NULL, thread_id);
-
+					    comm, my_rank, data_size, NULL, thread_id);
+  msg->root_process = root;
   __mpi_barrier_start_generic(msg, my_rank);
 }
 
@@ -1469,12 +1490,13 @@ __ezt_mpi_enter_function(MPI_ALLGATHER_ID);
   app_ptr comm_id;
   int my_rank;
   int comm_size;
-  GET_PARAM_PACKED_3(CUR_EV, comm_id, comm_size, my_rank);
+  int data_size;
+  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, data_size);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
   assert(comm);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_allgather,
-					    comm, my_rank, NULL, thread_id);
+					    comm, my_rank, data_size, NULL, thread_id);
 
   __mpi_barrier_start_generic(msg, my_rank);
 }
@@ -1494,11 +1516,12 @@ void handle_mpi_start_Alltoall()
   app_ptr comm_id;
   int my_rank;
   int comm_size;
-  GET_PARAM_PACKED_3(CUR_EV, comm_id, comm_size, my_rank);
+  int data_size;
+  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, data_size);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_alltoall,
-      comm, my_rank, NULL, thread_id);
+					    comm, my_rank, data_size, NULL, thread_id);
   __mpi_barrier_start_generic(msg, my_rank);
 }
 
@@ -1517,11 +1540,14 @@ void handle_mpi_start_Reduce()
   app_ptr comm_id;
   int my_rank;
   int comm_size;
-  GET_PARAM_PACKED_3(CUR_EV, comm_id, comm_size, my_rank);
+  int data_size;
+  int root;
+  GET_PARAM_PACKED_5(CUR_EV, comm_id, comm_size, my_rank, data_size, root);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_reduce,
-      comm, my_rank, NULL, thread_id);
+					    comm, my_rank, data_size, NULL, thread_id);
+  msg->root_process = root;
   __mpi_barrier_start_generic(msg, my_rank);
 }
 
@@ -1535,10 +1561,11 @@ void handle_mpi_start_Allreduce()
   app_ptr comm_id;
   int my_rank;
   int comm_size;
-  GET_PARAM_PACKED_3(CUR_EV, comm_id, comm_size, my_rank);
+  int data_size;
+  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, data_size);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_allreduce,
-      comm, my_rank, NULL, thread_id);
+					    comm, my_rank, data_size, NULL, thread_id);
   __mpi_barrier_start_generic(msg, my_rank);
 }
 
@@ -1552,11 +1579,12 @@ void handle_mpi_start_Reduce_scatter()
   app_ptr comm_id;
   int my_rank;
   int comm_size;
-  GET_PARAM_PACKED_3(CUR_EV, comm_id, comm_size, my_rank);
+  int data_size;
+  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, data_size);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_reduce_scatter,
-      comm, my_rank, NULL, thread_id);
+					    comm, my_rank, data_size, NULL, thread_id);
   __mpi_barrier_start_generic(msg, my_rank);
 }
 
@@ -1571,11 +1599,12 @@ void handle_mpi_start_Scan()
   app_ptr comm_id;
   int my_rank;
   int comm_size;
-  GET_PARAM_PACKED_3(CUR_EV, comm_id, comm_size, my_rank);
+  int data_size;
+  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, data_size);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_scan,
-      comm, my_rank, NULL, thread_id);
+					    comm, my_rank, data_size, NULL, thread_id);
   __mpi_barrier_start_generic(msg, my_rank);
 }
 
@@ -1593,7 +1622,7 @@ void handle_mpi_start_barrier()
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_barrier,
-      comm, my_rank, NULL, thread_id);
+					    comm, my_rank, 0, NULL, thread_id);
 
   __mpi_barrier_start_generic(msg, my_rank);
 }
@@ -1911,7 +1940,9 @@ void handle_mpi_Ibcast()
   int my_rank;
   int comm_size;
   app_ptr req;
-  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, req);
+  int data_size;
+  int root;
+  GET_PARAM_PACKED_6(CUR_EV, comm_id, comm_size, my_rank, req, data_size, root);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
 
   struct mpi_request * mpi_req = __mpi_new_mpi_request(CUR_RANK,
@@ -1920,7 +1951,8 @@ void handle_mpi_Ibcast()
   assert(mpi_req);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_bcast,
-      comm, my_rank, mpi_req, thread_id);
+					    comm, my_rank, data_size, mpi_req, thread_id);
+  msg->root_process = root;
   __mpi_barrier_start_generic(msg, my_rank);
   MPI_CHANGE() pushState (CURRENT, "ST_Thread", thread_id, "STV_MPI_Ibcast");
 }
@@ -1947,7 +1979,7 @@ void handle_mpi_Ibarrier()
   assert(mpi_req);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_barrier,
-					    comm, my_rank, mpi_req, thread_id);
+					    comm, my_rank, 0, mpi_req, thread_id);
   __mpi_barrier_start_generic(msg, my_rank);
   MPI_CHANGE() pushState (CURRENT, "ST_Thread", thread_id, "STV_MPI_Ibarrier");
 }
@@ -1962,7 +1994,9 @@ void handle_mpi_Igather()
   int my_rank;
   int comm_size;
   app_ptr req;
-  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, req);
+  int data_size;
+  int root;
+  GET_PARAM_PACKED_6(CUR_EV, comm_id, comm_size, my_rank, req, data_size, root);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
 
   struct mpi_request * mpi_req = __mpi_new_mpi_request(CUR_RANK,
@@ -1971,8 +2005,8 @@ void handle_mpi_Igather()
   assert(mpi_req);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_gather,
-					    comm, my_rank, mpi_req, thread_id);
-
+					    comm, my_rank, data_size, mpi_req, thread_id);
+  msg->root_process = root;
   __mpi_barrier_start_generic(msg, my_rank);
   MPI_CHANGE() pushState (CURRENT, "ST_Thread", thread_id, "STV_MPI_Igather");
 }
@@ -1991,10 +2025,10 @@ void handle_mpi_Iscatter()
   app_ptr comm_id;
   int my_rank;
   int comm_size;
-  int len;
+  int data_size;
   app_ptr req;
-
-  GET_PARAM_PACKED_5(CUR_EV, comm_id, comm_size, my_rank,len, req);
+  int root;
+  GET_PARAM_PACKED_6(CUR_EV, comm_id, comm_size, my_rank, req, data_size, root);
 
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
 
@@ -2004,8 +2038,9 @@ void handle_mpi_Iscatter()
   assert(mpi_req);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_scatter,
-					    comm, my_rank, mpi_req, thread_id);
+					    comm, my_rank, data_size, mpi_req, thread_id);
 
+  msg->root_process = root;
   __mpi_barrier_start_generic(msg, my_rank);
   MPI_CHANGE() pushState (CURRENT, "ST_Thread", thread_id, "STV_MPI_Iscatter");
 }
@@ -2025,7 +2060,8 @@ void handle_mpi_Iallgather()
   int my_rank;
   int comm_size;
   app_ptr req;
-  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, req);
+  int data_size;
+  GET_PARAM_PACKED_5(CUR_EV, comm_id, comm_size, my_rank, req, data_size);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
 
   struct mpi_request * mpi_req = __mpi_new_mpi_request(CUR_RANK,
@@ -2034,7 +2070,7 @@ void handle_mpi_Iallgather()
   assert(mpi_req);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_allgather,
-					    comm, my_rank, mpi_req, thread_id);
+					    comm, my_rank, data_size, mpi_req, thread_id);
 
   __mpi_barrier_start_generic(msg, my_rank);
   MPI_CHANGE() pushState (CURRENT, "ST_Thread", thread_id, "STV_MPI_Iallgather");
@@ -2055,7 +2091,8 @@ void handle_mpi_Ialltoall()
   int my_rank;
   int comm_size;
   app_ptr req;
-  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, req);
+  int data_size;
+  GET_PARAM_PACKED_5(CUR_EV, comm_id, comm_size, my_rank, req, data_size);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
   struct mpi_request * mpi_req = __mpi_new_mpi_request(CUR_RANK,
 						       req,
@@ -2063,7 +2100,7 @@ void handle_mpi_Ialltoall()
   assert(mpi_req);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_alltoall,
-					    comm, my_rank, mpi_req, thread_id);
+					    comm, my_rank, data_size, mpi_req, thread_id);
   __mpi_barrier_start_generic(msg, my_rank);
   MPI_CHANGE() pushState (CURRENT, "ST_Thread", thread_id, "STV_MPI_Ialltoall");
 }
@@ -2083,7 +2120,9 @@ void handle_mpi_Ireduce()
   int my_rank;
   int comm_size;
   app_ptr req;
-  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, req);
+  int data_size;
+  int root;
+  GET_PARAM_PACKED_6(CUR_EV, comm_id, comm_size, my_rank, req, data_size, root);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
   struct mpi_request * mpi_req = __mpi_new_mpi_request(CUR_RANK,
 						       req,
@@ -2091,8 +2130,8 @@ void handle_mpi_Ireduce()
   assert(mpi_req);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_reduce,
-      comm, my_rank, mpi_req, thread_id);
-
+					    comm, my_rank, data_size, mpi_req, thread_id);
+  msg->root_process = root;
   __mpi_barrier_start_generic(msg, my_rank);
   MPI_CHANGE() pushState (CURRENT, "ST_Thread", thread_id, "STV_MPI_Ireduce");
 }
@@ -2107,7 +2146,8 @@ void handle_mpi_Iallreduce()
   int my_rank;
   int comm_size;
   app_ptr req;
-  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, req);
+  int data_size;
+  GET_PARAM_PACKED_5(CUR_EV, comm_id, comm_size, my_rank, req, data_size);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
   struct mpi_request * mpi_req = __mpi_new_mpi_request(CUR_RANK,
 						       req,
@@ -2115,7 +2155,7 @@ void handle_mpi_Iallreduce()
   assert(mpi_req);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_allreduce,
-					    comm, my_rank, mpi_req, thread_id);
+					    comm, my_rank, data_size, mpi_req, thread_id);
   __mpi_barrier_start_generic(msg, my_rank);
   MPI_CHANGE() pushState (CURRENT, "ST_Thread", thread_id, "STV_MPI_Iallreduce");
 }
@@ -2130,7 +2170,8 @@ void handle_mpi_Ireduce_scatter()
   int my_rank;
   int comm_size;
   app_ptr req;
-  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, req);
+  int data_size;
+  GET_PARAM_PACKED_5(CUR_EV, comm_id, comm_size, my_rank, req, data_size);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
 
   struct mpi_request * mpi_req = __mpi_new_mpi_request(CUR_RANK,
@@ -2140,7 +2181,7 @@ void handle_mpi_Ireduce_scatter()
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX),
 					    mpi_coll_reduce_scatter, comm,
-					    my_rank, mpi_req, thread_id);
+					    my_rank, data_size, mpi_req, thread_id);
 
   __mpi_barrier_start_generic(msg, my_rank);
   MPI_CHANGE() pushState (CURRENT, "ST_Thread", thread_id, "STV_MPI_Ireduce_scatter");
@@ -2156,7 +2197,8 @@ void handle_mpi_Iscan()
   int my_rank;
   int comm_size;
   app_ptr req;
-  GET_PARAM_PACKED_4(CUR_EV, comm_id, comm_size, my_rank, req);
+  int data_size;
+  GET_PARAM_PACKED_5(CUR_EV, comm_id, comm_size, my_rank, req, data_size);
   struct ezt_mpi_comm* comm = ezt_find_communicator(CUR_INDEX, comm_id);
   struct mpi_request * mpi_req = __mpi_new_mpi_request(CUR_RANK,
 						       req,
@@ -2164,7 +2206,7 @@ void handle_mpi_Iscan()
   assert(mpi_req);
 
   struct mpi_coll_msg_t* msg = __enter_coll(CUR_TIME(CUR_INDEX), mpi_coll_scan,
-					    comm, my_rank, mpi_req, thread_id);
+					    comm, my_rank, data_size, mpi_req, thread_id);
 
   __mpi_barrier_start_generic(msg, my_rank);
   MPI_CHANGE() pushState (CURRENT, "ST_Thread", thread_id, "STV_MPI_Iscan");
@@ -2408,11 +2450,11 @@ void handle_mpi_start()
   if(req->type == mpi_req_send) {
   /* todo: mpi communicator */
     struct mpi_p2p_msg_t* msg = __mpi_send_generic( thread_id, CUR_RANK, req->dest, req->len, req->tag, req->mpi_req, p_info->__MPI_COMM_WORLD.comm_id);
-    if(!msg->times[stop_isend])
+    if(!IS_TIME_SET(msg->times[stop_isend]))
       msg->times[stop_isend] = CUR_TIME(CUR_INDEX);
   } else {
     struct mpi_p2p_msg_t* msg = __mpi_start_recv_generic( thread_id, req->src, CUR_RANK, req->len, req->tag, req->mpi_req, p_info->__MPI_COMM_WORLD.comm_id);
-    if(!msg->times[stop_irecv])
+    if(!IS_TIME_SET(msg->times[stop_irecv]))
       msg->times[stop_irecv] = CUR_TIME(CUR_INDEX);
   }
 
@@ -3054,11 +3096,8 @@ handle_mpi_events(eztrace_event_t *ev) {
 #endif
 
     printf("\n");
-    print_p2p_stats();
 
-    printf("\n");
-    mpi_stats_plot_message_size();
-    printf("\n");
+    print_mpi_msg_stats();
   }
 
 
