@@ -39,7 +39,9 @@ static void MPI_Iallreduce_prolog (CONST void *sendbuf __attribute__((unused)),
   int len;
   MPI_Type_size(datatype, &len);
 
-   EZTRACE_EVENT_PACKED_4(EZTRACE_MPI_IALLREDUCE, (app_ptr)comm, size, rank, (app_ptr)r);
+  int data_size = count * len;
+
+  EZTRACE_EVENT_PACKED_5(EZTRACE_MPI_IALLREDUCE, (app_ptr)comm, size, rank, (app_ptr)r, data_size);
 }
 
 static int MPI_Iallreduce_core (CONST void *sendbuf, void *recvbuf, int count,
