@@ -29,7 +29,7 @@ static void MPI_Igatherv_prolog(CONST void *sendbuf __attribute__((unused)),
 				CONST int *recvcnts __attribute__((unused)),
 				CONST int *displs __attribute__((unused)),
 				MPI_Datatype recvtype,
-				int root __attribute__((unused)),
+				int root,
 				MPI_Comm comm,
 				MPI_Request *r)
 {
@@ -38,7 +38,10 @@ static void MPI_Igatherv_prolog(CONST void *sendbuf __attribute__((unused)),
   libMPI_Comm_size(comm, &size);
   libMPI_Comm_rank(comm, &rank);
 
-  EZTRACE_EVENT_PACKED_4(EZTRACE_MPI_IGATHERV, (app_ptr)comm, size, rank, (app_ptr)r);
+  int ssize;
+  MPI_Type_size(sendtype, &ssize);
+  int data_size = ssize * sendcnt;
+  EZTRACE_EVENT_PACKED_6(EZTRACE_MPI_IGATHERV, (app_ptr)comm, size, rank, (app_ptr)r, data_size, root);
 }
 
 static int MPI_Igatherv_core(CONST void *sendbuf, int sendcnt, MPI_Datatype sendtype,
