@@ -23,8 +23,8 @@
 
 #ifdef USE_MPI3
 static void MPI_Iscan_prolog (CONST void *sendbuf __attribute__((unused)),
-			     void *recvbuf __attribute__((unused)),
-			     int count __attribute__((unused)),
+			      void *recvbuf __attribute__((unused)),
+			      int count __attribute__((unused)),
 			      MPI_Datatype datatype,
 			      MPI_Op op __attribute__((unused)),
 			      MPI_Comm comm,
@@ -35,10 +35,11 @@ static void MPI_Iscan_prolog (CONST void *sendbuf __attribute__((unused)),
   libMPI_Comm_size(comm, &size);
   libMPI_Comm_rank(comm, &rank);
 
-  int len;
-  MPI_Type_size(datatype, &len);
+  int ssize;
+  MPI_Type_size(datatype, &ssize);
+  int data_size = ssize * count;
 
-  EZTRACE_EVENT_PACKED_4(EZTRACE_MPI_ISCAN, (app_ptr)comm, size, rank, (app_ptr)r);
+  EZTRACE_EVENT_PACKED_5(EZTRACE_MPI_ISCAN, (app_ptr)comm, size, rank, (app_ptr)r, data_size);
 }
 
 static int MPI_Iscan_core (CONST void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype,
