@@ -11,7 +11,6 @@
  *  Created on: 2 juil. 2011
  *      Author: Damien Martin-Guillerez <damien.martin-guillerez@inria.fr>
  */
-
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/mman.h>
@@ -28,6 +27,14 @@
 #include <errno.h>
 #include <string.h>
 #include <stdarg.h>
+#include "linux.h"
+
+extern int pptrace_debug_level;
+#define _ptrace(req, pid, addr, data) __ptrace(req, pid, addr, data, __FILE__, __LINE__)
+static inline long __ptrace(enum __ptrace_request request, pid_t pid, void *addr,
+		     void *data, const char *file, int line);
+
+#if ENABLE_BINARY_INSTRUMENTATION
 
 #define CONTSIGNAL NULL
 static int ptrace_bits = __WORDSIZE; // Size of a word in the traced process
@@ -117,77 +124,7 @@ int trace_get_bits() {
   return ptrace_bits;
 }
 
-/*
- * A wrapper to test for errors
- * TODO: Correct error handling in that part of the code
- */
-static inline long __ptrace(enum __ptrace_request request, pid_t pid, void *addr,
-		     void *data, const char *file, int line) {
-  errno = 0;
-  long result = ptrace(request, pid, addr, data);
-  if (errno && result == -1) {
-    pptrace_fubar("at %s:%d: ptrace(%lu, %d, %p, %p): %s", file, line,
-		  (long) request, pid, addr, data, strerror(errno));
-  }
-  return result;
-}
-#define _ptrace(req, pid, addr, data) __ptrace(req, pid, addr, data, __FILE__, __LINE__)
 
-extern int pptrace_debug_level;
-
-pid_t trace_run(char *path, char **argv, char **envp, int debug) {
-  int status;
-  int fds[2];
-  pipe(fds);
-  char plop;
-  pid_t child = fork();
-
-  if (pptrace_debug_level > PPTRACE_DEBUG_LEVEL_ALL)
-    debug = 1;
-
-  if ((debug == 0 && child != 0) || (debug && child == 0)) {
-#ifdef __PPTRACE_USE_PRCTL
-#ifdef PR_SET_PTRACER
-    prctl(PR_SET_PTRACER, child, 0, 0, 0);
-#endif // PR_SET_PTRACER
-#endif // __PPTRACE_USE_PRCTL
-    if (child == 0)
-      _ptrace(PTRACE_TRACEME, 0, NULL, NULL);
-
-    read(fds[0], &plop, sizeof(char));
-    close(fds[0]);
-    if (path == NULL) { // No execution is asked (fork only) so return to the calling function
-      kill(getpid(), SIGTRAP); // signal the parent to emulate execve call.
-      return 0;
-    }
-    pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "execve(%s, ...)\n", path);
-    execve(path, argv, envp);
-    pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE, "Execution of %s failed\n",
-		  path);
-    exit(-1);
-  }
-  pid_t tpid = debug ? child : getppid();
-  if (debug == 0)
-    _ptrace(PTRACE_ATTACH, tpid, NULL, NULL);
-  plop = 'a';
-  write(fds[1], &plop, sizeof(char));
-  close(fds[1]);
-  waitpid(tpid, &status, 0);
-  while (!WIFEXITED(status)
-    && (!WIFSTOPPED(status) || WSTOPSIG(status) != SIGTRAP)) {
-    _ptrace(PTRACE_CONT, tpid, NULL, NULL);
-    waitpid(tpid, &status, 0);
-  }
-  pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "execve(%s, ...) passed\n", path);
-  if (WIFEXITED(status)) {
-    pptrace_debug(
-	PPTRACE_DEBUG_LEVEL_VERBOSE,
-	"Process %d exited (probably due to execution of %s failed)\n", tpid,
-	path);
-    return -1;
-  }
-  return tpid;
-}
 
 word_uint get_ip(pid_t child) {
   REG_STRUCT regs;
@@ -409,144 +346,6 @@ void print_reg_value(pid_t child, word_uint addr) {
 printf("[" FORMAT_WORD_HEX "] from address " FORMAT_WORD_POINTER "\n", ARG_FORMAT_WORD(data), addr);
 }
 
-word_uint trace_mmap(pid_t child, word_uint addr, size_t length, int prot) {
-int i;
-// Save user registers
-REG_STRUCT regs;
-_ptrace(PTRACE_GETREGS, child, NULL, &regs);
-REG_STRUCT saved_regs;
-_ptrace(PTRACE_GETREGS, child, NULL, &saved_regs);
-
-#if __WORDSIZE == 64
-if(ptrace_bits == 32) {
-#endif
-
-#if __arm__
-// ARMv7
-/*
- Linux syscall on 32-bits ARMv7 (svc #0 = 00 EF) number __NR_mmap (90)
- r7 = __NR_mmap
- r0 = arg1 => addr
- r1 = arg2 => length
- r2 = arg3 => prot
- r3 = arg4 => flags = MAP_ANONYMOUS | MAP_PRIVATE
- sp+12 = arg5 => fd = -1
- sp+16 = arg6 => offset = 0
- return r0 = return value = addr
- */
-
-word_uint prev_val1 = -1;
-word_uint prev_val2 = -1;
-
-regs.REG_R7 = 192; // __NR_mmap2 32-bits
-regs.REG_R0 = addr;
-regs.REG_R1 = length;
-regs.REG_R2 = prot;
-regs.REG_R3 = MAP_ANONYMOUS | MAP_PRIVATE;
-
-// backup the values from sp
-prev_val1 = do_push(child, (word_uint *) &saved_regs.REG_SP, -1);
-prev_val2 = do_push(child, (word_uint *) &saved_regs.REG_SP, 0);
-
-#elif __x86_64__
-// x86_64
-/*
- Linux syscall on 32-bits (int 0x80 = CD 80) number __NR_mmap (90)
- eax = __NR_mmap
- ebx = arg1 => addr
- ecx = arg2 => length
- edx = arg3 => prot
- esi = arg4 => flags = MAP_ANONYMOUS | MAP_PRIVATE
- edi = arg5 => fd = -1
- ebp = arg6 => offset = 0
- return rax = return value = addr
- */
-regs.REG_AX = 192; // __NR_mmap2 32-bits
-regs.REG_BX = addr;
-regs.REG_CX = length;
-regs.REG_DX = prot;
-regs.REG_SI = MAP_ANONYMOUS | MAP_PRIVATE;
-regs.REG_DI = -1;
-regs.REG_BP = 0;
-#endif
-#if __WORDSIZE == 64
-}
-else {
-/*
- Linux syscall on 64-bits (syscall - 0f 05) number __NR_mmap (9)
- rax = __NR_mmap
- rdi = arg1 => addr
- rsi = arg2 => length
- rdx = arg3 => prot
- r10 = arg4 => flags = MAP_ANONYMOUS | MAP_PRIVATE | MAP_32BIT
- r8 = arg5 => fd = -1
- r9 = arg6 => offset = 0
- return rax = return value = addr
- */
-regs.REG_AX = 9; // __NR_mmap 64-bits
-regs.REG_DI = addr;
-regs.REG_SI = length;
-regs.REG_DX = prot;
-regs.REG_R10 = MAP_ANONYMOUS | MAP_PRIVATE | MAP_32BIT;
-regs.REG_R8 = -1;
-regs.REG_R9 = 0;
-}
-#endif
-// Pushing new user registers
-_ptrace(PTRACE_SETREGS, child, NULL, &regs);
-
-#if __arm__
-// ARMv7
-// Inserting svc #0 (00 00 00 ef)
-uint8_t opcode[] = {0x00, 0x00, 0x00, 0xef};
-int size = 4;
-#elif __x86_64__
-// x86_64
-// Inserting syscall (0f 05) / int 0x80 (cd 80)
-uint8_t opcode[] = {0x0f, 0x05, 0xcd, 0x80};
-int off = ptrace_bits == 32 ? 2 : 0;
-int size = 2;
-#endif
-for (i = 0; i < size; i++)
-#if __arm__
-// ARMv7
-  opcode[i] = trace_replace(child, saved_regs.REG_PC + i, opcode[i]);
-#elif __x86_64__
-// x86_64
-  opcode[i + off] = trace_replace(child, saved_regs.REG_IP + i, opcode[i + off]);
-#endif
-
-  // Two step execution with PTRACE_SYSCALL to return
-  _ptrace(PTRACE_SYSCALL, child, NULL, NULL);
-waitpid(child, NULL, 0); // The syscall was called
-_ptrace(PTRACE_SYSCALL, child, NULL, NULL);
-waitpid(child, NULL, 0); // The syscall has returned
-
-// Getting back the result from the syscall
-_ptrace(PTRACE_GETREGS, child, NULL, &regs);
-
-// Inserting back the original opcode
-for (i = 0; i < size; i++)
-#if __arm__
-// ARMv7
-  opcode[i] = trace_replace(child, saved_regs.REG_PC + i, opcode[i]);
-#elif __x86_64__
-// x86_64
-  opcode[i + off] = trace_replace(child, saved_regs.REG_IP + i, opcode[i + off]);
-#endif
-
-#if __arm__
-  /* restore the values that were overwritten by do_push */
-  do_pop(child, (word_uint *) &saved_regs.REG_SP, prev_val2);
-  do_pop(child, (word_uint *) &saved_regs.REG_SP, prev_val1);
-#endif
-
-  // Restoring registers
-  _ptrace(PTRACE_SETREGS, child, NULL, &saved_regs);
-
-// Returning the result of the syscall
-return get_retval(regs);
-}
 
 char *trace_get_zstring(pid_t child, word_uint addr) {
 unsigned int size = 1024;
@@ -767,13 +566,232 @@ pptrace_debug(PPTRACE_DEBUG_LEVEL_DEBUG,
 return -1;
 }
 
-void trace_detach(pid_t child) {
-_ptrace(PTRACE_DETACH, child, NULL, NULL);
+word_uint trace_mmap(pid_t child, word_uint addr, size_t length, int prot) {
+int i;
+// Save user registers
+REG_STRUCT regs;
+_ptrace(PTRACE_GETREGS, child, NULL, &regs);
+REG_STRUCT saved_regs;
+_ptrace(PTRACE_GETREGS, child, NULL, &saved_regs);
+
+#if __WORDSIZE == 64
+if(ptrace_bits == 32) {
+#endif
+
+#if __arm__
+// ARMv7
+/*
+ Linux syscall on 32-bits ARMv7 (svc #0 = 00 EF) number __NR_mmap (90)
+ r7 = __NR_mmap
+ r0 = arg1 => addr
+ r1 = arg2 => length
+ r2 = arg3 => prot
+ r3 = arg4 => flags = MAP_ANONYMOUS | MAP_PRIVATE
+ sp+12 = arg5 => fd = -1
+ sp+16 = arg6 => offset = 0
+ return r0 = return value = addr
+ */
+
+word_uint prev_val1 = -1;
+word_uint prev_val2 = -1;
+
+regs.REG_R7 = 192; // __NR_mmap2 32-bits
+regs.REG_R0 = addr;
+regs.REG_R1 = length;
+regs.REG_R2 = prot;
+regs.REG_R3 = MAP_ANONYMOUS | MAP_PRIVATE;
+
+// backup the values from sp
+prev_val1 = do_push(child, (word_uint *) &saved_regs.REG_SP, -1);
+prev_val2 = do_push(child, (word_uint *) &saved_regs.REG_SP, 0);
+
+#elif __x86_64__
+// x86_64
+/*
+ Linux syscall on 32-bits (int 0x80 = CD 80) number __NR_mmap (90)
+ eax = __NR_mmap
+ ebx = arg1 => addr
+ ecx = arg2 => length
+ edx = arg3 => prot
+ esi = arg4 => flags = MAP_ANONYMOUS | MAP_PRIVATE
+ edi = arg5 => fd = -1
+ ebp = arg6 => offset = 0
+ return rax = return value = addr
+ */
+regs.REG_AX = 192; // __NR_mmap2 32-bits
+regs.REG_BX = addr;
+regs.REG_CX = length;
+regs.REG_DX = prot;
+regs.REG_SI = MAP_ANONYMOUS | MAP_PRIVATE;
+regs.REG_DI = -1;
+regs.REG_BP = 0;
+#endif
+#if __WORDSIZE == 64
+}
+else {
+/*
+ Linux syscall on 64-bits (syscall - 0f 05) number __NR_mmap (9)
+ rax = __NR_mmap
+ rdi = arg1 => addr
+ rsi = arg2 => length
+ rdx = arg3 => prot
+ r10 = arg4 => flags = MAP_ANONYMOUS | MAP_PRIVATE | MAP_32BIT
+ r8 = arg5 => fd = -1
+ r9 = arg6 => offset = 0
+ return rax = return value = addr
+ */
+regs.REG_AX = 9; // __NR_mmap 64-bits
+regs.REG_DI = addr;
+regs.REG_SI = length;
+regs.REG_DX = prot;
+regs.REG_R10 = MAP_ANONYMOUS | MAP_PRIVATE | MAP_32BIT;
+regs.REG_R8 = -1;
+regs.REG_R9 = 0;
+}
+#endif
+// Pushing new user registers
+_ptrace(PTRACE_SETREGS, child, NULL, &regs);
+
+#if __arm__
+// ARMv7
+// Inserting svc #0 (00 00 00 ef)
+uint8_t opcode[] = {0x00, 0x00, 0x00, 0xef};
+int size = 4;
+#elif __x86_64__
+// x86_64
+// Inserting syscall (0f 05) / int 0x80 (cd 80)
+uint8_t opcode[] = {0x0f, 0x05, 0xcd, 0x80};
+int off = ptrace_bits == 32 ? 2 : 0;
+int size = 2;
+#endif
+for (i = 0; i < size; i++)
+#if __arm__
+// ARMv7
+  opcode[i] = trace_replace(child, saved_regs.REG_PC + i, opcode[i]);
+#elif __x86_64__
+// x86_64
+  opcode[i + off] = trace_replace(child, saved_regs.REG_IP + i, opcode[i + off]);
+#endif
+
+  // Two step execution with PTRACE_SYSCALL to return
+  _ptrace(PTRACE_SYSCALL, child, NULL, NULL);
+waitpid(child, NULL, 0); // The syscall was called
+_ptrace(PTRACE_SYSCALL, child, NULL, NULL);
+waitpid(child, NULL, 0); // The syscall has returned
+
+// Getting back the result from the syscall
+_ptrace(PTRACE_GETREGS, child, NULL, &regs);
+
+// Inserting back the original opcode
+for (i = 0; i < size; i++)
+#if __arm__
+// ARMv7
+  opcode[i] = trace_replace(child, saved_regs.REG_PC + i, opcode[i]);
+#elif __x86_64__
+// x86_64
+  opcode[i + off] = trace_replace(child, saved_regs.REG_IP + i, opcode[i + off]);
+#endif
+
+#if __arm__
+  /* restore the values that were overwritten by do_push */
+  do_pop(child, (word_uint *) &saved_regs.REG_SP, prev_val2);
+  do_pop(child, (word_uint *) &saved_regs.REG_SP, prev_val1);
+#endif
+
+  // Restoring registers
+  _ptrace(PTRACE_SETREGS, child, NULL, &saved_regs);
+
+// Returning the result of the syscall
+return get_retval(regs);
 }
 
-void trace_wait(pid_t child) {
-int status = 0;
-do {
-  waitpid(child, &status, 0);
-} while (!WIFEXITED(status) && !WIFSIGNALED(status));
+#endif
+
+pid_t trace_run(char *path, char **argv, char **envp, int debug) {
+  int status;
+  int fds[2];
+  pipe(fds);
+  char plop;
+  pid_t child = fork();
+
+  if (pptrace_debug_level > PPTRACE_DEBUG_LEVEL_ALL)
+    debug = 1;
+
+  if ((debug == 0 && child != 0) || (debug && child == 0)) {
+#ifdef __PPTRACE_USE_PRCTL
+#ifdef PR_SET_PTRACER
+    prctl(PR_SET_PTRACER, child, 0, 0, 0);
+#endif // PR_SET_PTRACER
+#endif // __PPTRACE_USE_PRCTL
+    if (child == 0)
+      _ptrace(PTRACE_TRACEME, 0, NULL, NULL);
+
+    /* wait until the parent process is attached */
+    read(fds[0], &plop, sizeof(char));
+    close(fds[0]);
+    if (path == NULL) { // No execution is asked (fork only) so return to the calling function
+      kill(getpid(), SIGTRAP); // signal the parent to emulate execve call.
+      return 0;
+    }
+    /* run the application */
+    pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "execve(%s, ...)\n", path);
+    execve(path, argv, envp);
+    pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE, "Execution of %s failed\n",
+		  path);
+    exit(-1);
+  }
+  pid_t tpid = debug ? child : getppid();
+  if (debug == 0)
+    _ptrace(PTRACE_ATTACH, tpid, NULL, NULL);
+
+  /* notify the child process that we are attached */
+  plop = 'a';
+  write(fds[1], &plop, sizeof(char));
+  close(fds[1]);
+  waitpid(tpid, &status, 0);
+
+  while (!WIFEXITED(status)
+    && (!WIFSTOPPED(status) || WSTOPSIG(status) != SIGTRAP)) {
+    _ptrace(PTRACE_CONT, tpid, NULL, NULL);
+    waitpid(tpid, &status, 0);
+  }
+  pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "execve(%s, ...) passed\n", path);
+
+  if (WIFEXITED(status)) {
+    pptrace_debug(
+	PPTRACE_DEBUG_LEVEL_VERBOSE,
+	"Process %d exited (probably due to execution of %s failed)\n", tpid,
+	path);
+    return -1;
+  }
+
+  return tpid;
 }
+
+void trace_detach(pid_t child) {
+  _ptrace(PTRACE_DETACH, child, NULL, NULL);
+}
+
+
+void trace_wait(pid_t child) {
+  int status = 0;
+  do {
+    waitpid(child, &status, 0);
+  } while (!WIFEXITED(status) && !WIFSIGNALED(status));
+}
+
+/*
+ * A wrapper to test for errors
+ * TODO: Correct error handling in that part of the code
+ */
+static inline long __ptrace(enum __ptrace_request request, pid_t pid, void *addr,
+		     void *data, const char *file, int line) {
+  errno = 0;
+  long result = ptrace(request, pid, addr, data);
+  if (errno && result == -1) {
+    pptrace_fubar("at %s:%d: ptrace(%lu, %d, %p, %p): %s", file, line,
+		  (long) request, pid, addr, data, strerror(errno));
+  }
+  return result;
+}
+
