@@ -58,6 +58,7 @@ typedef struct __pptrace_internal_binary {
   void *binary;  // Binary structure
 
   char **debugger; // Argument list to debugger an argument matching "{name}" will be replaced by the binary name and arguments matching "{pid}" will be replaced by the program pid
+  zzt_word baseaddr;		/* base address of the binary */
   // List of libraries
   pptrace_internal_library* first;
   pptrace_internal_library* last;
@@ -123,6 +124,7 @@ void* pptrace_prepare_binary(char *binary) {
     pptrace_error("Allocation failed");
     return NULL;
   }
+#if ENABLE_BINARY_INSTRUMENTATION
   bin->binary = open_binary(bin->name);
   if (!(bin->binary)) {
     pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "failed!\n");
@@ -131,6 +133,7 @@ void* pptrace_prepare_binary(char *binary) {
     free(bin);
     return NULL;
   }
+#endif
 
   pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "ok\n");
   return (void*) bin;
@@ -148,7 +151,7 @@ void pptrace_add_debugger(void *bin, char **argv) {
 void pptrace_free_hijack(pptrace_internal_hijack* hijack) {
   if (!hijack)
     return;
-
+#if ENABLE_BINARY_INSTRUMENTATION
   if (hijack->origSym)
     free_symbol(hijack->origSym);
   if (hijack->replSym)
@@ -162,13 +165,14 @@ void pptrace_free_hijack(pptrace_internal_hijack* hijack) {
     free(hijack->replace_symbol);
   if (hijack->function)
     free(hijack->function);
-
+#endif
   free(hijack);
 }
 
 pptrace_internal_hijack* pptrace_get_hijack(pptrace_internal_binary *bin,
 					    char *library, void *lib,
 					    char *params) {
+#if ENABLE_BINARY_INSTRUMENTATION
   char *params_copy;
   char *function;
   char *interceptee;
@@ -223,6 +227,7 @@ pptrace_internal_hijack* pptrace_get_hijack(pptrace_internal_binary *bin,
   }
   free(params_copy);
 
+  /* search for the address of the symbol */
   hijack->funSym = get_symbol(bin->binary, hijack->function);
   if (!hijack->funSym) {
     pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO,
@@ -249,6 +254,8 @@ pptrace_internal_hijack* pptrace_get_hijack(pptrace_internal_binary *bin,
   }
 
   return hijack;
+#endif
+  return NULL;
 }
 
 void pptrace_free_hijacks(pptrace_internal_hijack** hijacks) {
@@ -308,6 +315,7 @@ void __pptrace_get_module_name(const char* libname,
 
 pptrace_internal_hijack** pptrace_load_hijacks(pptrace_internal_binary *bin,
 					       char *library, void *lib) {
+#if ENABLE_BINARY_INSTRUMENTATION
   void *bin_bin = open_binary(library);
   char module_name[1024];
   __pptrace_get_module_name(library, module_name, 1024);
@@ -335,6 +343,8 @@ pptrace_internal_hijack** pptrace_load_hijacks(pptrace_internal_binary *bin,
 							 params);
   free_zstring_array(params);
   return result;
+#endif
+  return NULL;
 }
 
 void pptrace_free_library(pptrace_internal_library *lib) {
@@ -379,10 +389,12 @@ int pptrace_load_module(void *bin, char *library) {
   if (r < 0)
     return r;
 
+#if ENABLE_BINARY_INSTRUMENTATION
   pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "Fetching hijacks of library %s\n",
 		library);
   pptrace_internal_binary* binary = (pptrace_internal_binary*) bin;
   pptrace_internal_library* lib = binary->last;
+
   void *libr = open_binary(library);
   if (!binary) {
     pptrace_remove_last_library(binary);
@@ -401,6 +413,7 @@ int pptrace_load_module(void *bin, char *library) {
 
   pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "Hijacks of library %s fetched\n",
 		library);
+#endif
   return 0;
 }
 
@@ -469,6 +482,7 @@ char* pptrace_create_preload(pptrace_internal_library *first) {
   return result;
 }
 
+/* preload the libraries, and load the application */
 pid_t pptrace_launch_preload(char *path, char **argv, char **envp,
 			     pptrace_internal_library *first, int debug) {
   // Adding LD_PRELOAD=libraries in envp
@@ -495,9 +509,11 @@ void pptrace_free_binary(pptrace_internal_binary* binary) {
   if (!binary)
     return;
 
+#if ENABLE_BINARY_INSTRUMENTATION
   if (binary->binary) {
     close_binary(binary->binary);
   }
+#endif
   while (binary->first) {
     lib = binary->first;
     binary->first = lib->next;
@@ -548,8 +564,32 @@ void pptrace_apply_library_baseaddr(pptrace_internal_library *lib,
   }
 }
 
+/* search for the base address of binary (once it is loaded) and fill
+ * the baseaddr field of the binary structure
+ */
+int pptrace_get_base_address(pid_t child, pptrace_internal_binary* binary) {
+  binary->baseaddr = 0;
+  /* the binary is mapped at this address:
+     grep "<binary->name>$" /proc/<child>/maps  |grep "r-xp"|cut -d'-' -f1
+  */
+  char cmd[4096];
+  snprintf(cmd, 4096, "grep \"$(basename %s)$\" /proc/%d/maps  |grep \" r-xp \"|cut -d'-' -f1", binary->name, child);
+  FILE*f = popen(cmd, "r");
+  if(!f) {
+    fprintf(stderr, "popen(%s) failed\n", cmd);
+    return -1;
+  }
+  uint64_t ptr;
+  fscanf(f, "%lx", &binary->baseaddr);
+  pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE,
+		"Binary base address: %p\n", binary->baseaddr);
+  return 0;
+}
+
 int pptrace_wait_library(pid_t child, pptrace_internal_binary* binary) {
+#if ENABLE_BINARY_INSTRUMENTATION
   char *library;
+
   // Wait for dlopen(argv[1])
   pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE,
 		"Waiting for open(*, O_RDONLY) (syscall %d)... ",
@@ -605,6 +645,7 @@ int pptrace_wait_library(pid_t child, pptrace_internal_binary* binary) {
   }
   pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE, "ok (addr = %016lx)\n", baseaddr);
   pptrace_apply_library_baseaddr(lib, (zzt_word) baseaddr);
+#endif	/* ENABLE_BINARY_INSTRUMENTATION */
   return 0;
 }
 
@@ -621,14 +662,23 @@ int pptrace_wait_libraries(pid_t child, pptrace_internal_binary* binary) {
  * or -1 if an error occured.
  */
 int pptrace_install_hijacks(pid_t child, pptrace_internal_binary* binary) {
+  int nb_installed = 0;
+#if ENABLE_BINARY_INSTRUMENTATION
   int i;
   pptrace_internal_library* lib;
-  int nb_installed = 0;
   for (lib = binary->first; lib != NULL; lib = lib->next) {
     if (lib->hijacks) {
       for (i = 0; lib->hijacks[i] != NULL; i++) {
 	if (lib->hijacks[i]->funSym != NULL
 	  && lib->hijacks[i]->funSym->symbol_offset) {
+
+	  if(lib->hijacks[i]->funSym->flags & ZZT_FLAG_DYNAMIC) {
+	    /* the symbol is in a relocated binary (ie. compiled with
+	     * -fPIE. We need to shift the base address.
+	     */
+	    lib->hijacks[i]->funSym->section_addr = lib->hijacks[i]->funSym->section_addr + binary->baseaddr;
+	  }
+
 	  // Replacing symbol only when the symbol is not in a dynamic library
 	  if (hijack(binary->binary, child, lib->hijacks[i]->funSym,
 		     lib->hijacks[i]->origSym, lib->hijacks[i]->replSym) < 0) {
@@ -644,6 +694,7 @@ int pptrace_install_hijacks(pid_t child, pptrace_internal_binary* binary) {
       }
     }
   }
+#endif
   return nb_installed;
 }
 
@@ -655,7 +706,9 @@ int pptrace_run(void *bin, char **argv, char **envp) {
     return -1;
   }
   pptrace_internal_binary* binary = (pptrace_internal_binary*) bin;
+#if ENABLE_BINARY_INSTRUMENTATION
   trace_set_bits(get_binary_bits(binary->binary));
+#endif
   pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "Running binary %s... ",
 		binary->name);
   pid_t child = pptrace_launch_preload(binary->name, argv, envp, binary->first,
@@ -668,6 +721,13 @@ int pptrace_run(void *bin, char **argv, char **envp) {
   }
   pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "ok (pid = %d)\n", child);
 
+  pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "Getting the binary base address\n");
+  if (pptrace_get_base_address(child, binary)) {
+    pptrace_error("Failed to get the base address of the binary, exiting after detaching child process...");
+    pptrace_free_binary(binary);
+    return -1;
+  }
+
   pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "Getting libraries' addresses\n");
   if (pptrace_wait_libraries(child, binary)) {
     pptrace_error(
